@@ -91,6 +91,8 @@ GPParams gp_params;
 /* flag for SIGUSR1 handler */
 static volatile int capture_now = 0;
 
+/* flag for SIGUSR2 handler */
+static volatile int end_next = 0;
 
 /*! \brief Copy string almost like strncpy, converting to lower case.
  *
@@ -590,8 +592,15 @@ sig_handler_capture_now (int sig_num)
 	capture_now = 1;
 }
 
+static void
+sig_handler_end_next (int sig_num)
+{
+        signal (SIGUSR2, sig_handler_end_next);
+        end_next = 1;
+}
+
 int
-capture_generic (CameraCaptureType type, const char __unused__ *name)
+capture_generic (CameraCaptureType type, const char __unused__ *name, int download)
 {
 	CameraFilePath path, last;
 	char *pathsep;
@@ -612,6 +621,8 @@ capture_generic (CameraCaptureType type, const char __unused__ *name)
 	}
 	capture_now = 0;
 	signal(SIGUSR1, sig_handler_capture_now);
+	end_next = 0;
+	signal(SIGUSR2, sig_handler_end_next);
 
 	while(++frames) {
 		if (!(gp_params.flags & FLAGS_QUIET) && glob_interval) {
@@ -651,64 +662,50 @@ capture_generic (CameraCaptureType type, const char __unused__ *name)
 			printf (_("New file is in location %s%s%s on the camera\n"),
 				path.folder, pathsep, path.name);
 
-		if(!glob_interval) break;
+		if (download) {
+			if (strcmp(path.folder, last.folder)) {
+				memcpy(&last, &path, sizeof(last));
 
-		if(strcmp(path.folder, last.folder)) {
-			memcpy(&last, &path, sizeof(last));
+				result = set_folder_action(&gp_params, path.folder);
+				if (result != GP_OK) {
+					cli_error_print(_("Could not set folder."));
+					return (result);
+				}
+			}
 
-			result = set_folder_action(&gp_params, path.folder);
+			result = get_file_common (path.name, GP_FILE_TYPE_NORMAL);
 			if (result != GP_OK) {
-				cli_error_print(_("Could not set folder."));
+				cli_error_print (_("Could not get image."));
+				if(result == GP_ERROR_FILE_NOT_FOUND) {
+					/* Buggy libcanon.so?
+					 * Can happen if this was the first capture after a
+					 * CF card format, or during a directory roll-over,
+					 * ie: CANON100 -> CANON101
+					 */
+					cli_error_print ( _("Buggy libcanon.so?"));
+				}
+				return (result);
+			}
+
+			if (!(gp_params.flags & FLAGS_QUIET))
+				printf (_("Deleting file %s%s%s on the camera\n"),
+					path.folder, pathsep, path.name);
+
+			result = delete_file_action (&gp_params, path.name);
+			if (result != GP_OK) {
+				cli_error_print ( _("Could not delete image."));
 				return (result);
 			}
 		}
-
-		/* XXX: I wonder if there is some way to determine if the camera
-		 * is still writing the image because if we don't sleep here,
-		 * the image will be corrupt when we attempt to download it.
-		 * I assume this is a result of the camera still writing the
-		 * data!  I picked 2 seconds as a first try and it seems like
-		 * enough time for high-quality JPEG on the Canon EOS 10D (with
-		 * a San Disk Ultra II CF).  May need to be increased for RAW
-		 * images...  I don't think this is the correct way to do this.
-		 */
-
-		/* Marcus Meissner: Fix the camera driver to only return from
-		 * camera_capture() when you know it is done ...
-		 * Otherwise other drivers suffer.
-		 */
-#if 0
-		sleep(2);
-#endif
-
-		result = get_file_common (path.name, GP_FILE_TYPE_NORMAL);
-		if (result != GP_OK) {
-			cli_error_print (_("Could not get image."));
-			if(result == GP_ERROR_FILE_NOT_FOUND) {
-				/* Buggy libcanon.so?
-				 * Can happen if this was the first capture after a
-				 * CF card format, or during a directory roll-over,
-				 * ie: CANON100 -> CANON101
-				 */
-				cli_error_print ( _("Buggy libcanon.so?"));
-			}
-			return (result);
-		}
-
-		if (!(gp_params.flags & FLAGS_QUIET))
-			printf (_("Deleting file %s%s%s on the camera\n"),
-				path.folder, pathsep, path.name);
-
-		result = delete_file_action (&gp_params, path.name);
-		if (result != GP_OK) {
-			cli_error_print ( _("Could not delete image."));
-			return (result);
-		}
-
 		/* Break if we've reached the requested number of frames
 		 * to capture.
 		 */
+		if(!glob_interval) break;
+
 		if(glob_frames && frames == glob_frames) break;
+		
+		/* Break if we've been told to end before the next frame */
+		if(end_next) break;
 
 #if 0
 		/* Marcus Meissner: Before you enable this, try to fix the
@@ -805,7 +802,7 @@ capture_tethered (const char __unused__ *name)
 			break;
 		case GP_EVENT_FILE_ADDED:
 			fn = (CameraFilePath*)data;
-			printf( _("New file %s/%s, downloading...\n"),fn->name, fn->folder);
+			printf( _("New file %s/%s, downloading...\n"),fn->folder, fn->name);
 
 			if(strcmp(fn->folder, last.folder)) {
 				strcpy(last.folder, fn->folder);
@@ -979,6 +976,7 @@ typedef enum {
 	ARG_CAPTURE_FRAMES,
 	ARG_CAPTURE_INTERVAL,
 	ARG_CAPTURE_IMAGE,
+	ARG_CAPTURE_IMAGE_AND_DOWNLOAD,
 	ARG_CAPTURE_MOVIE,
 	ARG_CAPTURE_PREVIEW,
 	ARG_CAPTURE_SOUND,
@@ -1236,16 +1234,19 @@ cb_arg_run (poptContext __unused__ ctx,
 		params->p.r = auto_detect_action (&gp_params);
 		break;
 	case ARG_CAPTURE_IMAGE:
-		params->p.r = capture_generic (GP_CAPTURE_IMAGE, arg);
+		params->p.r = capture_generic (GP_CAPTURE_IMAGE, arg, 0);
+		break;
+	case ARG_CAPTURE_IMAGE_AND_DOWNLOAD:
+		params->p.r = capture_generic (GP_CAPTURE_IMAGE, arg, 1);
 		break;
 	case ARG_CAPTURE_MOVIE:
-		params->p.r = capture_generic (GP_CAPTURE_MOVIE, arg);
+		params->p.r = capture_generic (GP_CAPTURE_MOVIE, arg, 0);
 		break;
 	case ARG_CAPTURE_PREVIEW:
 		params->p.r = action_camera_capture_preview (&gp_params);
 		break;
 	case ARG_CAPTURE_SOUND:
-		params->p.r = capture_generic (GP_CAPTURE_SOUND, arg);
+		params->p.r = capture_generic (GP_CAPTURE_SOUND, arg, 0);
 		break;
 	case ARG_CAPTURE_TETHERED:
 		params->p.r = capture_tethered (arg);
@@ -1467,7 +1468,7 @@ report_failure (int result, int argc, char **argv)
 		 */
 		printf ("    env LANG=C gphoto2 --debug --debug-logfile=my-logfile.txt");
 		for (n = 1; n < argc; n++) {
-			if (argv[n][0] == '-')
+			if (strchr(argv[n], ' ') == NULL)
 				printf(" %s",argv[n]);
 			else
 				printf(" \"%s\"",argv[n]);
@@ -1607,6 +1608,8 @@ main (int argc, char **argv, char **envp)
 		 N_("Reset capture interval on signal (default=no)"), NULL},
 		{"capture-image", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_IMAGE, N_("Capture an image"), NULL},
+		{"capture-image-and-download", '\0', POPT_ARG_NONE, NULL,
+		 ARG_CAPTURE_IMAGE_AND_DOWNLOAD, N_("Capture an image and download it"), NULL},
 		{"capture-movie", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_MOVIE, N_("Capture a movie"), NULL},
 		{"capture-sound", '\0', POPT_ARG_NONE, NULL,
@@ -1736,7 +1739,9 @@ main (int argc, char **argv, char **envp)
 
 	/* Figure out the width of the terminal and watch out for changes */
 	signal_resize (0);
+#ifdef SIGWINCH
 	signal (SIGWINCH, signal_resize);
+#endif
 
 	/* Prepare processing options. */
 	ctx = poptGetContext (PACKAGE, argc, (const char **) argv, options, 0);
@@ -1865,6 +1870,7 @@ main (int argc, char **argv, char **envp)
 		gp_log (GP_LOG_DEBUG, "main", "The user has not specified "
 			"both a model and a port. Try to figure them out.");
 
+
 		_get_portinfo_list(&gp_params);
 		CR_MAIN (gp_list_new (&list)); /* no freeing below */
 		CR_MAIN (gp_abilities_list_detect (gp_params_abilities_list(&gp_params), 
@@ -1902,7 +1908,7 @@ main (int argc, char **argv, char **envp)
                 } else if (!count) {
 			/*
 			 * No camera detected. Have a look at the settings.
-			 * Ignore errors here.
+			 * Ignore errors here, it might be a serial one.
 			 */
                         if (gp_setting_get ("gphoto2", "model", buf) >= 0)
 				action_camera_set_model (&gp_params, buf);
@@ -1910,11 +1916,34 @@ main (int argc, char **argv, char **envp)
 				action_camera_set_port (&gp_params, buf);
 
 		} else {
+			/* If --port override, search the model with the same port. */
+			if (info.type != GP_PORT_NONE) {
+				int i;
+				gp_log (GP_LOG_DEBUG, "gphoto2", "Looking for port ...\n");
+				gp_log (GP_LOG_DEBUG, "gphoto2", "info.type = %d\n", info.type);
+				gp_log (GP_LOG_DEBUG, "gphoto2", "info.name = %s\n", info.name);
+				gp_log (GP_LOG_DEBUG, "gphoto2", "info.path = %s\n", info.path);
+
+				for (i=0;i<count;i++) {
+					const char *xport, *xmodel;
+					gp_list_get_value (list, i, &xport);
+					if (!strcmp(xport, info.path)) {
+						gp_list_get_name (list, i, &xmodel);
+						CR_MAIN (action_camera_set_model (&gp_params, xmodel));
+						CR_MAIN (action_camera_set_port (&gp_params, xport));
+						gp_log (GP_LOG_DEBUG, "gphoto2","found port, was entry %d\n", i);
+						break;
+					}
+				}
+				if (i != count)
+					use_auto = 0;
+			}
 			/* If --camera override, search the model with the same USB ID. */
-			if (a.model[0]) {
+			if (use_auto && a.model[0]) {
 				int i;
 				const char *xmodel;
 
+				gp_log (GP_LOG_DEBUG, "gphoto2","Looking for model %s\n", a.model);
 				for (i=0;i<count;i++) {
 					CameraAbilities alt;
 					int m;
@@ -1940,6 +1969,7 @@ main (int argc, char **argv, char **envp)
 			if (use_auto) {
 				/* More than one camera detected */
 				/*FIXME: Let the user choose from the list!*/
+				gp_log (GP_LOG_DEBUG,"gphoto2","Nothing specified, using first entry of autodetect list.\n");
 				CR_MAIN (gp_list_get_name (list, 0, &model));
 				CR_MAIN (gp_list_get_value (list, 0, &path));
 				CR_MAIN (action_camera_set_model (&gp_params, model));
